Attribute VB_Name = "CD_AutoDesign"
Option Private Module
Option Explicit

Private Const HEADS_UNQ_PREF = " _MKT_DLT_HEAD_"
Private Const LISTS_STARTS = " _MKT_DLT_LIST_"

Private Const TABLE_WIDTH_LIMIT As Double = 8
Private Const TABLE_HEADER_STYLE = " "

Private Const STYLE_CONTRACT = "_ "
Private Const STYLE_THEME = "_    "
Private Const STYLE_REQUIREMENTS = "_  "

Public Function RunAutoDesign(prefs As AutoDesignOptions)
  Dim source As Word.Document:  Set source = ActiveDocument

  Call CSE_ProgressBar.Init("  ", maxVal:=11 + prefs.Count)
  Call CSE_ProgressBar.ShowModeless
  
  Dim proxyCount&: proxyCount = ADS_PrepareSource(source)
  Dim target As Word.Document: Set target = ADS_CreateTarget
  
  Dim chapters As Collection:  Set chapters = ADS_GenerateSections(target, source)
  Dim listStyles As Collection:  Set listStyles = ADS_FillSections(target, source, chapters)
  
  Call ADS_SetPageSettings(target)
  
  If prefs.fixObjects Then _
    Call ADS_TablesAndInlines(target)
  If prefs.reapplyLists Then _
    Call ADS_ReapplyStyles(target, listStyles)
  
  Call ADS_FormatHeaders(target, source, proxyCount)
  
  If prefs.titlePage Then _
    Call ADS_TitlePage(target, source)
  If prefs.generateColontitles Then _
    Call ADS_Colontitles(target)

  If prefs.doLayout Then _
    Call ADS_TextLayout(target)
    
  Call ADS_UpdateToc(target)
  Call ADS_Paint(target)
  
  Call ADS_Finalize(target)

  Unload CSE_ProgressBar
End Function

' ==============
Private Function ADS_UpdateToc(target As Word.Document)
  CSE_ProgressBar.Header = " ..."
  Dim toc As Word.TableOfContents
  For Each toc In target.TablesOfContents
    Call UpdateTableOfContents(toc)
  Next toc
  Call CSE_ProgressBar.IncrementA
End Function

Private Function ADS_Paint(target As Word.Document)
  Call RepaintText(target)
  Call CSE_ProgressBar.IncrementA
  
  Call RepaintTextShapes(target)
  Call CSE_ProgressBar.IncrementA
  
  Call RepaintToC(target)
  Call CSE_ProgressBar.IncrementA
  
  Call RepaintHLinks(target)
  Call CSE_ProgressBar.IncrementA
End Function

Private Function ADS_PrepareSource(source As Word.Document) As Long
  CSE_ProgressBar.Header = "  ..."
  ADS_PrepareSource = MarkListItems(source)
  Call MarkListsBegin(source)
  Call CSE_ProgressBar.IncrementA
End Function

Private Function ADS_CreateTarget() As Word.Document
  CSE_ProgressBar.Header = " ..."
  
  Dim wrapper As New API_WordWrapper
  Dim target As Word.Document: Set target = wrapper.NewDocument("20 ", False)
  target.AttachedTemplate = ""
  target.UpdateStylesOnOpen = False
  
  Call RemoveSampleSections(target)
  
  Set ADS_CreateTarget = target
  Call CSE_ProgressBar.IncrementA
End Function

Private Function ADS_GenerateSections(target As Word.Document, source As Word.Document) As Collection
  CSE_ProgressBar.Header = " ..."
  Dim chapters As Collection: Set chapters = ScanChaptersInfo(source)
  Dim intro As Word.Range: Set intro = target.Range(target.Sections(4).Range.Start, target.Sections(6).Range.End)
  Call CopyPasteRepeat(intro, target.Sections(7).Range.Start, target, chapters.Count - 1)
  
  Call TransferChapterHeaders(chapters, target)
  
  Call CSE_ProgressBar.IncrementA
  Set ADS_GenerateSections = chapters
End Function

Private Function ADS_FillSections(target As Word.Document, source As Word.Document, chapters As Collection) As Collection
  CSE_ProgressBar.Header = " ..."
  
  Dim listStyles As New Collection
  Dim aStyle As Word.Style
  For Each aStyle In source.Styles
    Dim sLocal$: sLocal = aStyle.NameLocal
    If WordStyleExists(target, sLocal) Then
      If CheckListStyle(target.Styles(sLocal)) Then
        Call listStyles.Add(sLocal)
      End If
    End If
  Next aStyle
  
  Call TransferChapersBody(chapters, source, target)
  
  Call CSE_ProgressBar.IncrementA
  Set ADS_FillSections = listStyles
End Function

Private Function ADS_SetPageSettings(target As Word.Document)
  CSE_ProgressBar.Header = "  ..."
  Call CSE_ProgressBar.InitSecondBar(0, target.Sections.Count - 3)
  Dim i&
  For i = 3 To target.Sections.Count - 1
    With target.Sections(i).PageSetup
      .LeftMargin = CentimetersToPoints(FIELD_SIZE_CM)
      .RightMargin = CentimetersToPoints(FIELD_SIZE_CM)
      .TopMargin = CentimetersToPoints(FIELD_SIZE_CM)
      .BottomMargin = CentimetersToPoints(FIELD_SIZE_CM)
      
      .DifferentFirstPageHeaderFooter = False
      .HeaderDistance = CentimetersToPoints(1.25)
      .FooterDistance = CentimetersToPoints(1.25)
      .OddAndEvenPagesHeaderFooter = False
    End With
    Call CSE_ProgressBar.IncrementB
  Next i
  
  target.Footnotes.Location = wdBeneathText
  
  Call CSE_ProgressBar.IncrementA
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function ADS_TablesAndInlines(target As Word.Document)
  CSE_ProgressBar.Header = " ..."
  Call CSE_ProgressBar.InitSecondBar(0, target.InlineShapes.Count + target.Tables.Count)

  Dim iShp As Word.InlineShape
  For Each iShp In target.InlineShapes
    Dim rShape As Word.Range: Set rShape = iShp.Range
    If rShape.PageSetup.TextColumns.Count > 1 Then
      If rShape.Paragraphs(1).Next.Range.Style = " " Then
        Call rShape.MoveEnd(wdParagraph, 2)
        Call InsertOneColomnSection(rShape)
      End If
    End If
    
    Call CSE_ProgressBar.IncrementB
  Next iShp
  
  Dim aTable As Word.Table
  For Each aTable In target.Tables
    Dim rTable As Word.Range: Set rTable = aTable.Range
    aTable.PreferredWidthType = wdPreferredWidthPoints
    If rTable.PageSetup.TextColumns.Count > 1 And aTable.PreferredWidth >= CentimetersToPoints(TABLE_WIDTH_LIMIT) Then
      aTable.PreferredWidthType = wdPreferredWidthPercent
      aTable.PreferredWidth = 100
      If rTable.Paragraphs(1).Previous.Range.Style = TABLE_HEADER_STYLE Then
        Call rTable.MoveStart(wdParagraph, -1)
        Call InsertOneColomnSection(rTable)
      End If
    End If
    
    Call CSE_ProgressBar.IncrementB
  Next aTable
  
  Call CSE_ProgressBar.IncrementA
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function ADS_ReapplyStyles(target As Word.Document, listStyles As Collection)
  CSE_ProgressBar.Header = "  ..."
    
  Call CSE_ProgressBar.InitSecondBar(0, listStyles.Count)
  Dim aStyle As Variant
  For Each aStyle In listStyles
    Call ReApplyStyle(target, CStr(aStyle))
    Call CSE_ProgressBar.IncrementB
  Next aStyle
  
  Call CSE_ProgressBar.IncrementA
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function ADS_FormatHeaders(target As Word.Document, source As Word.Document, proxyCount&)
  CSE_ProgressBar.Header = " ..."
  
  Call FinalizeLists(source, True)
  Call FinalizeLists(target, False)
  
  Call FixGapsStyle(target)
  Call TransferHeaderFormats(target, source)
  
  Call RemoveListPoxy(source, proxyCount)
  Call RemoveListPoxy(target, proxyCount)
  
  Call CSE_ProgressBar.IncrementA
End Function

Private Function ADS_TitlePage(target As Word.Document, source As Word.Document)
  CSE_ProgressBar.Header = "    ..."
  Call TransferTitlePage(source, target)
  Call TransferWorkerList(source, target)
  Call TransferRequirements(source, target)
  Call CSE_ProgressBar.IncrementA
End Function

Private Function ADS_TextLayout(target As Word.Document)
  CSE_ProgressBar.Header = "..."
  Call UpdateListsLayout(target)
  Call UpdateTextLayout(target)
  Call UpdateObjectFields(target)
  Call CSE_ProgressBar.IncrementA
End Function

Private Function ADS_Colontitles(target As Word.Document)
  CSE_ProgressBar.Header = " ..."
  
  Dim docData As New InfoDocument: Call docData.Init(target)
  Dim iColons As New ItemColontitles
  With iColons
    .mTopLeft = T_SOURCE_VOLUME
    .mTopRight = T_SOURCE_BOOK
    .mBottomLeft = T_SOURCE_CONCEPT
    .mBottomRight = T_SOURCE_SECTION
    .start_ = 3
    .finish_ = target.Sections.Count - 1
    
    .doBottomRight = True
    .doTopLeft = docData.IsValidNames
    .doTopRight = .doTopLeft
  End With
  
  Call CSE_ProgressBar.InitSecondBar(0, iColons.finish_ - iColons.start_ + 1)
  Call CreateColontitles(target, docData, iColons, "IncrementB")
  Call CSE_ProgressBar.HideSecondBar
  Call CSE_ProgressBar.IncrementA
End Function

Private Function ADS_Finalize(target As Word.Document)
  CSE_ProgressBar.Header = "..."
  Dim i%
  For i = 3 To target.Sections.Count - 1
    With target.Sections(i)
      Dim iComment As Word.Range: Set iComment = target.Range(.Range.Start, .Range.Start)
      If .PageSetup.Orientation = wdOrientLandscape Then _
        Call iComment.Comments.Add(iComment, "   ?")
    End With
  Next i
  Call CSE_ProgressBar.IncrementA
End Function

Private Function RemoveSampleSections(target As Word.Document)
  Do While target.Sections.Count > 7
    Call target.Sections(target.Sections.Count - 1).Range.Delete
  Loop
End Function

Private Function FinalizeLists(target As Word.Document, justClear As Boolean)
  Dim rFind As Word.Range: Set rFind = target.Range
  rFind.Find.Text = LISTS_STARTS
  Do While rFind.Find.Execute
    Call rFind.MoveEnd(wdCharacter, 4)
    Dim aTemplate As Word.ListTemplate: Set aTemplate = rFind.ListFormat.ListTemplate
    aTemplate.ListLevels(1).StartAt = CLng(Right(rFind.Text, 4))
    If Not justClear Then
      Call rFind.ListFormat.ApplyListTemplate(ListTemplate:=aTemplate, _
                                              ContinuePreviousList:=False, _
                                              ApplyTo:=wdListApplyToWholeList, _
                                              DefaultListBehavior:=wdWord10ListBehavior)
    End If
    rFind.Delete
  Loop
End Function

Private Function MarkListsBegin(target As Word.Document)
  Dim tmpLst As Word.List, tmpLstRange As Word.Range
  For Each tmpLst In target.Lists
    If IsHeader(tmpLst.ListParagraphs(1).Range) Then _
      GoTo CONT_FOR
    Set tmpLstRange = tmpLst.ListParagraphs(1).Range
    
    Call tmpLstRange.MoveEndUntil(Chr(13), wdBackward)
    Call tmpLstRange.MoveEnd(wdCharacter, -1)
    
    If Right(tmpLstRange.ListFormat.ListString, 2) Like "#?" Then
      Call tmpLstRange.InsertAfter(LISTS_STARTS & Format(tmpLstRange.ListFormat.ListValue, "0000"))
    End If
CONT_FOR:
  Next tmpLst
End Function

Private Function MarkListItems(target As Word.Document) As Long
  Dim hdNames As New Collection
  Dim listItem As Word.Paragraph
  For Each listItem In target.ListParagraphs
    Dim tmpRng As Word.Range: Set tmpRng = listItem.Range
    If Not InCollection(tmpRng.Text, hdNames) Then
      Call hdNames.Add(tmpRng.Text, tmpRng.Text)
      GoTo NEXT_ITEM
    End If

    MarkListItems = MarkListItems + 1
    Call tmpRng.MoveEnd(wdCharacter, -1)
    Dim sMark$: sMark = HEADS_UNQ_PREF & CStr(MarkListItems)
    Call tmpRng.InsertAfter(sMark)
    Call tmpRng.MoveEnd(wdCharacter, 1)
NEXT_ITEM:
  Next listItem
End Function

Private Function RemoveListPoxy(target As Word.Document, proxyCount&)
  Dim nProxy&
  For nProxy = 1 To proxyCount Step 1
    Dim rFind As Word.Range: Set rFind = target.Range
    With rFind.Find
      .ClearFormatting
      .Text = "<" & Mid(HEADS_UNQ_PREF & nProxy, 3) & ">"
      .MatchWildcards = True
      
      If .Execute Then
        Call rFind.MoveStart(wdCharacter, -2)
        rFind.Delete
      End If
    End With
  Next nProxy
End Function

Private Function CheckListStyle(target As Word.Style) As Boolean
  CheckListStyle = Not target.NameLocal Like "[]*"
  CheckListStyle = CheckListStyle And _
    (target.NameLocal Like "*[][]*" Or target.NameLocal Like "*[]*")
End Function

Private Function TransferWorkerList(source As Word.Document, dest As Word.Document)
  source.Sections(3).Range.Tables(1).Range.Copy
  Dim insertPosition&: insertPosition = dest.Sections(2).Range.Tables(1).Range.Start
  Call dest.Sections(2).Range.Tables(1).Delete
  Call dest.Range(insertPosition&, insertPosition&).PasteAndFormat(wdUseDestinationStylesRecovery)
End Function

Private Function TransferRequirements(source As Word.Document, dest As Word.Document)
  If RequirementsPosition(source) = 0 Then _
    Exit Function
  If RequirementsPosition(dest) = 0 Then _
    Exit Function
    
  Call source.Tables(source.Tables.Count).Range.Copy
  Call dest.Tables(dest.Tables.Count).Delete
  Call dest.Paragraphs.Last.Range.Paste
End Function

Private Function TransferChapterHeaders(chapters As Collection, doc As Word.Document)
  Call CSE_ProgressBar.InitSecondBar(0, chapters.Count)
  
  Dim ColorIndex&: ColorIndex = 0
  Dim i&, j&: j = 4
  For i = 1 To chapters.Count
    Dim headerText$: headerText$ = chapters(i).text_
    headerText = Left(headerText$, Len(headerText$) - 1)
    
    Dim rPaste As Word.Range: Set rPaste = doc.Sections(j).Range.Duplicate
    Call rPaste.MoveEnd(wdWord, -2)
    rPaste.Text = headerText$
    
    Dim clrID As WdThemeColorIndex
    Select Case (i - 1 - ColorIndex) Mod 7
      Case 0 To 4: clrID = (i - 1 - ColorIndex) Mod 7 + 5
      Case 5: clrID = wdThemeColorBackground2
      Case 6: clrID = wdThemeColorText2
    End Select
    
    If InStr(UCase(headerText), "") + InStr(UCase(headerText), "") <> 0 Then
      clrID = wdThemeColorAccent1
      ColorIndex = ColorIndex + 1
    End If
    rPaste.Paragraphs.First.Range.Font.Color = DesignTheme(clrID)
    j = j + 3
    
    Call CSE_ProgressBar.IncrementB
  Next i
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function ReApplyStyle(target As Word.Document, aStyle$)
  If Not WordStyleExists(target, aStyle) Then _
    Exit Function
  
  Dim rFind As Word.Range: Set rFind = target.Range
  Dim lastEnd&
  rFind.Find.Style = target.Styles(aStyle)
  Do While rFind.Find.Execute
    Dim resetRange As Word.Range: Set resetRange = rFind.Duplicate
    resetRange.Start = IIf(Len(rFind) < 2, rFind.Start, rFind.End - 1)
    resetRange.End = IIf(Len(rFind) < 2, rFind.End, rFind.End - 1)
    resetRange.Style = target.Styles(aStyle)
    
    Call rFind.Collapse(wdCollapseEnd)
    If rFind.Start >= target.Range.End - 1 Then _
      Exit Function
    If rFind.Start = lastEnd Then _
      Call rFind.Move(wdCharacter, 1)
    
    lastEnd = rFind.End
  Loop
End Function

Private Function ScanChaptersInfo(target As Word.Document) As Collection
  Dim result As New Collection
  Dim para As Word.Paragraph
  For Each para In target.Paragraphs
    If IsHeader(para.Range, 1) Then
      Dim info As New ItemChapter
      info.start_ = para.Range.Start
      info.finish_ = para.Range.End
      
      Dim lstStrng$: lstStrng = ""
      If para.Range.ListParagraphs.Count <> 0 Then _
        lstStrng = para.Range.ListFormat.ListString
      
      info.text_ = lstStrng & _
            IIf(Left(para.Range, 1) = " ", "", _
              IIf(lstStrng = "", "", " ")) & para.Range
      Call result.Add(info.Clone)
    End If
  Next para
  Set ScanChaptersInfo = result
End Function

Private Function TransferChapersBody(chapters As Collection, source As Word.Document, dest As Word.Document)
  Call CSE_ProgressBar.InitSecondBar(0, chapters.Count)
  
  Dim aChapter As ItemChapter: Set aChapter = chapters(chapters.Count)
  Dim lastFinish&: lastFinish = RequirementsPosition(source)
  If aChapter.finish_ <= lastFinish Then
    Call source.Range(aChapter.finish_, lastFinish).Copy
    Call dest.Range(dest.Sections(chapters.Count * 3 + 2).Range.Start, dest.Sections(chapters.Count * 3 + 2).Range.Start).Paste
  End If
  Call CSE_ProgressBar.IncrementB
  
  Dim i&
  For i = chapters.Count - 1 To 1 Step -1
    If chapters(i).finish_ <> chapters(i + 1).start_ Then
      Call source.Range(chapters(i).finish_, chapters(i + 1).start_).Copy
      Call dest.Range(dest.Sections(i * 3 + 2).Range.Start, dest.Sections(i * 3 + 2).Range.Start).Paste
    End If
    Call CSE_ProgressBar.IncrementB
  Next i
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function TransferHeaderFormats(target As Word.Document, source As Word.Document)
  Call CSE_ProgressBar.InitSecondBar(0, target.ListParagraphs.Count)
  
  Dim listPrefixes As New Collection: Set listPrefixes = ExtractListPrefixes(source)
  
  Dim pCurrent As Word.Paragraph
  For Each pCurrent In target.ListParagraphs
    Dim rCurrent As Word.Range: Set rCurrent = pCurrent.Range
    If rCurrent.ParagraphFormat.OutlineLevel <> wdOutlineLevelBodyText Then
      Dim sPrefix$: sPrefix = listPrefixes(rCurrent.Text)
      Dim sHeaderText$: sHeaderText = RemoveListPrefix(rCurrent.Text)
      Call InsertHeader(rCurrent, sPrefix & " " & sHeaderText, rCurrent.ListFormat.ListLevelNumber)
    End If
    
    Call CSE_ProgressBar.IncrementB
  Next pCurrent
  
  For Each pCurrent In target.ListParagraphs
    If pCurrent.Range.ParagraphFormat.OutlineLevel <> wdOutlineLevelBodyText Then _
      Call pCurrent.Range.Delete
  Next pCurrent
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Private Function TransferTitlePage(source As Word.Document, target As Word.Document)
  Dim titleData As TitlePageData: titleData = ExtractTitlePageData(source)

  Call TransferBook(target, titleData)
  Call TransferVolume(target, titleData)
  
  If Not titleData.rContract Is Nothing Then _
    Call TransferContract(target, titleData.rContract)
  If Not titleData.rCustomer Is Nothing Then _
    Call TransferCustomer(target, titleData.rCustomer)
  If Not titleData.rTheme Is Nothing Then _
    Call TransferTheme(target, titleData.rTheme)
End Function

Private Function CopyPasteRepeat(target As Word.Range, nPos&, doc As Word.Document, nCount&)
  Call target.Copy
  Dim i&
  For i = 1 To nCount
    Call doc.Range(nPos, nPos).PasteAndFormat(wdFormatOriginalFormatting)
  Next i
End Function

Private Function ExtractListPrefixes(target As Word.Document) As Collection
  Dim result As New Collection
  Dim listItem As Word.Paragraph
  For Each listItem In target.ListParagraphs
    If listItem.Range.ParagraphFormat.OutlineLevel <> wdOutlineLevelBodyText Then
      Call result.Add(listItem.Range.ListFormat.ListString, listItem.Range.Text)
    End If
  Next listItem
  Set ExtractListPrefixes = result
End Function

Private Function RemoveListPrefix(listText$) As String
  Dim result$: result = listText
  Do While Asc(Right(result, 1)) <= 32
    result = Left(result, Len(result) - 1)
  Loop
  RemoveListPrefix = result
End Function

Private Function RequirementsPosition(target As Word.Document) As Long
  RequirementsPosition = target.Range.End
  Dim rFind As Word.Range: Set rFind = target.Range
  With rFind.Find
    .Text = " "
    .Style = STYLE_REQUIREMENTS
    If Not .Execute Then _
      Exit Function
  End With
  RequirementsPosition = rFind.Start
End Function

Private Function ExtractTitlePageData(target As Word.Document) As TitlePageData
  Dim result As TitlePageData
  
  Dim rFind As Word.Range
  Set rFind = target.Sections(2).Range
  rFind.Find.Text = " "
  rFind.Find.MatchCase = False
  If rFind.Find.Execute Then
    Set rFind = rFind.Next(wdWord, 1)
    result.nBookID = CLng(rFind.Text)
    
    Call rFind.MoveEndUntil(CSET_SLETTERS & CSET_BLETTERS, wdForward)
    Call rFind.Collapse(wdCollapseEnd)
    Call rFind.MoveEndUntil(Chr(13), wdForward)
    result.sBook = rFind.Text
  End If
  
  Set rFind = target.Sections(2).Range
  rFind.Find.Text = " "
  rFind.Find.MatchCase = False
  If rFind.Find.Execute Then
    Set rFind = rFind.Next(wdWord, 1)
    result.nVolumeID = CLng(rFind.Text)
    
    Call rFind.MoveEndUntil(CSET_SLETTERS & CSET_BLETTERS, wdForward)
    Call rFind.Collapse(wdCollapseEnd)
    Call rFind.MoveEndUntil(Chr(13))
    result.sVolume = rFind.Text
  End If
  
  Set rFind = target.Sections(2).Range
  rFind.Find.Text = " "
  rFind.Find.Style = STYLE_CONTRACT
  rFind.Find.Format = True
  If rFind.Find.Execute Then
    Set result.rContract = rFind.Paragraphs.First.Range.Duplicate
  End If
  
  Set rFind = target.Sections(2).Range
  rFind.Find.Text = ": "
  rFind.Find.Style = STYLE_CONTRACT
  rFind.Find.Format = True
  If rFind.Find.Execute Then
    Set result.rCustomer = rFind.Paragraphs.First.Range.Duplicate
  End If

  Set rFind = target.Sections(2).Range
  rFind.Find.Style = STYLE_THEME
  rFind.Find.Format = True
  If rFind.Find.Execute Then
    Set result.rTheme = rFind.Paragraphs.First.Range.Duplicate
  End If
  
  ExtractTitlePageData = result
End Function

Private Function TransferBook(target As Word.Document, titleData As TitlePageData)
  Dim rPaste As Word.Range: Set rPaste = target.Sections(1).Range
  With rPaste.Find
    .Text = " "
    .Format = True
    .MatchCase = False
  End With
  
  If Not rPaste.Find.Execute Then _
    Exit Function
  Set rPaste = rPaste.Next(wdWord, 1)
  If titleData.nBookID <> 0 Then _
    rPaste.Text = titleData.nBookID
    
  If titleData.sBook = "" Then _
    Exit Function
  Call rPaste.MoveEndUntil(CSET_SLETTERS & CSET_BLETTERS, wdForward)
  Call rPaste.Collapse(wdCollapseEnd)
  Call rPaste.MoveEndUntil(Chr(13), wdForward)
  rPaste.Text = titleData.sBook
End Function

Private Function TransferVolume(target As Word.Document, titleData As TitlePageData)
  Dim rPaste As Word.Range: Set rPaste = target.Sections(1).Range
  With rPaste.Find
    .Text = " "
    .Format = True
    .MatchCase = False
  End With
  If Not rPaste.Find.Execute Then _
    Exit Function
    
  Set rPaste = rPaste.Next(wdWord, 1)
  If titleData.nVolumeID <> 0 Then _
    rPaste.Text = titleData.nVolumeID
    
  If titleData.sVolume = "" Then _
    Exit Function
  Call rPaste.MoveEndUntil(CSET_SLETTERS & CSET_BLETTERS, wdForward)
  Call rPaste.Collapse(wdCollapseEnd)
  Call rPaste.MoveEndUntil(Chr(13), wdForward)
  rPaste.Text = titleData.sVolume
End Function

Private Function TransferContract(target As Word.Document, rContract As Word.Range)
  Dim rPaste As Word.Range: Set rPaste = target.Sections(1).Range
  With rPaste.Find
    .Text = " "
    .Format = True
    .Style = STYLE_CONTRACT
  End With
  If Not rPaste.Find.Execute Then _
    Exit Function
  
  Call rContract.Copy
  Call rPaste.Paragraphs.First.Range.PasteAndFormat(wdUseDestinationStylesRecovery)
End Function

Private Function TransferCustomer(target As Word.Document, rCustomer As Word.Range)
  Dim rPaste As Word.Range: Set rPaste = target.Sections(1).Range
  With rPaste.Find
    .Text = " "
    .Format = True
    .Style = STYLE_CONTRACT
  End With
  If Not rPaste.Find.Execute Then _
    Exit Function
  
  Call rCustomer.Copy
  Call rPaste.Paragraphs.First.Range.PasteAndFormat(wdUseDestinationStylesRecovery)
End Function

Private Function TransferTheme(target As Word.Document, rTheme As Word.Range)
  Dim rPaste As Word.Range: Set rPaste = target.Sections(1).Range
  With rPaste.Find
    .Format = True
    .Style = STYLE_THEME
  End With
  If Not rPaste.Find.Execute Then _
    Exit Function
  
  Call rTheme.Copy
  Call rPaste.Paragraphs.First.Range.PasteAndFormat(wdUseDestinationStylesRecovery)
End Function
