Attribute VB_Name = "CD_Layout"
Option Private Module
Option Explicit

Public Function InsertOneColomnSection(target As Word.Range)
  Dim nEnd&: nEnd = target.End
  Call target.InsertBreak(wdSectionBreakContinuous)
  Call target.Document.Range(nEnd + 1, nEnd + 1).InsertBreak(wdSectionBreakContinuous)
  Call target.Sections(1).PageSetup.TextColumns.SetCount(1)
End Function

Public Function InsertHeader(dest As Word.Range, headerName$, nLvl&)
  Dim headerText$: headerText = Replace(headerName, Chr(11), " ")
  headerText = Replace(headerText, "  ", " ")
  
  Dim prefixPosition&: prefixPosition = 1
  Do While Not Mid(headerText, prefixPosition, 1) Like "[--a-zA-Z]"
    prefixPosition = prefixPosition + 1
  Loop
  
  Dim headerPrefix$
  If prefixPosition > 1 Then headerPrefix = Left(headerText, prefixPosition - 2)
  headerText = Right(headerText, Len(headerText) - prefixPosition + 1)
  
  Call dest.InsertAfter(headerText & vbNewLine)
  Dim insRange As Word.Range: Set insRange = dest.Paragraphs.Last.Range
  insRange.Font.Reset
  insRange.Style = " " & Trim(Str(nLvl))
  insRange.Font.Color = GetSectionHeader(insRange.Start, dest.Document).Font.Color
  
  If prefixPosition = 1 Then _
    Exit Function
  CreateEmptyField(insRange).TextFrame.TextRange.Text = headerPrefix
End Function

Public Function InsertConceptSymbol(dest As Word.Range, symbol$)
  If symbol = "" Then _
    Exit Function
    
  Dim txtShp As Word.Shape
  Set txtShp = CreateEmptyField(dest)
  With txtShp.TextFrame.TextRange
    .Font.Spacing = 0
    .ParagraphFormat.LineSpacingRule = wdLineSpaceSingle
    .Font.Size = CentimetersToPoints(1.08)
    .Text = symbol
    .Font.Color = GetSectionHeader(dest.Start, dest.Document).Font.Color
    .Font.Name = "conceptpict"
  End With
  txtShp.TextFrame.AutoSize = True
End Function

Public Function InsertNewPicture(sFile$, dest As Word.Range)
  Dim tmpRange As Word.Range: Set tmpRange = dest.Application.Selection.Range
  
  Dim txtShp As Word.Shape: Set txtShp = CreateEmptyField(dest)
  txtShp.TextFrame.TextRange.Select
  Dim picShp As Word.InlineShape: Set picShp = Selection.InlineShapes.AddPicture(sFile)
  If picShp Is Nothing Then
    Call UserInteraction.ShowMessage(EM_CANNOT_INSERT_IMAGE)
    Exit Function
  End If
  txtShp.TextFrame.TextRange.ParagraphFormat.LineSpacingRule = wdLineSpaceSingle
  
  picShp.LockAspectRatio = True
  picShp.Width = CentimetersToPoints(1)
  txtShp.TextFrame.AutoSize = True
  
  Dim sPictName$: sPictName = Replace(UserInteraction.PromptInput("  , :" & vbNewLine & "Pict_99"), " ", "_")
  Dim sBookmark$: sBookmark = "pict_" & sPictName
  
  On Error GoTo INVALID_NAME
TRY_AGAIN:
  Call dest.Document.Bookmarks.Add(sBookmark, txtShp.TextFrame.TextRange)
  Call tmpRange.Select
  Exit Function
  
INVALID_NAME:
  sBookmark = "pict_" & dest.Document.Bookmarks.Count
  Call UserInteraction.ShowMessage(IM_FIX_BOOKMARK_NAME, sBookmark)
  GoTo TRY_AGAIN
End Function

Public Function InsertPictureRef(dest As Word.Range, bookmarkID$)
  Dim txtShp As Word.Shape:  Set txtShp = CreateEmptyField(dest)
  With txtShp
    With .TextFrame.TextRange
      .ParagraphFormat.LineSpacingRule = wdLineSpaceSingle
      Call .Fields.Add(txtShp.TextFrame.TextRange, Text:="REF " & bookmarkID)
    End With
    .TextFrame.AutoSize = True
  End With
End Function

Public Function InsertTextField(dest As Word.Range, insText$)
  Call dest.Collapse(wdCollapseStart)
  Call dest.MoveEnd(wdCharacter, 1)
  CreateEmptyField(dest).TextFrame.TextRange.Text = insText
End Function

Public Function InlineAsPNG(target As Word.InlineShape)
  Dim isVisio As Boolean: isVisio = False
  Dim prevW As Double: prevW = 0
  If Not target.OLEFormat Is Nothing Then
    If InStr(UCase(target.OLEFormat.ClassType), "VISIO") <> 0 Then
      isVisio = True
      prevW = target.Width
      
      Dim formFactor As Double: formFactor = target.Height / target.Width
      target.LockAspectRatio = msoFalse
      target.Width = CentimetersToPoints(20)
      target.Height = target.Width * formFactor
    End If
  End If
  
  Dim dest As Word.Range: Set dest = target.Range.Duplicate
  target.Select
  target.Application.Selection.Cut
  With dest
    Call .Collapse(wdCollapseEnd)
    Call .PasteSpecial(Placement:=wdInLine, DataType:=IIf(isVisio, wdPasteEnhancedMetafile, 14))
    If isVisio Then
      Call .MoveStart(wdCharacter, -1)
      If .InlineShapes(1).Width < prevW Then
        .InlineShapes(1).LockAspectRatio = msoTrue
        .InlineShapes(1).Width = prevW
        Call InlineAsPNG(.InlineShapes(1))
      Else
        Call InlineAsPNG(.InlineShapes(1))
        Call .MoveEnd(wdCharacter, 1)
        .InlineShapes(1).LockAspectRatio = msoTrue
        .InlineShapes(1).Width = prevW
      End If
    End If
  End With
End Function

Public Function UpdateListsLayout(target As Word.Document)
  Call CSE_ProgressBar.InitSecondBar(0, target.ListParagraphs.Count)
  
  Dim aParagraph As Word.Paragraph
  For Each aParagraph In target.ListParagraphs
    If aParagraph.Range = target.Paragraphs.Last.Range Then _
      GoTo NEXT_LIST
    If aParagraph.Range.Tables.Count > 0 Then _
      GoTo NEXT_LIST
    
    Dim rNext As Word.Range: Set rNext = aParagraph.Next.Range
    If InStr(rNext.Text, Chr(12)) Or InStr(rNext.Text, Chr(14)) Then _
      GoTo NEXT_LIST
    If rNext.ListParagraphs.Count <> 0 Then _
      GoTo NEXT_LIST
      
    Dim rCurrent As Word.Range: Set rCurrent = aParagraph.Range
    With rCurrent.ParagraphFormat
      .SpaceAfter = .SpaceAfter - (Int(.SpaceAfter / .LineSpacing) - 1) * .LineSpacing
    End With
    
NEXT_LIST:
    Call CSE_ProgressBar.IncrementB
  Next aParagraph
  Call CSE_ProgressBar.HideSecondBar
End Function

Public Function UpdateTextLayout(target As Word.Document)
  Call FixGapsStyle(target)
  
  Call CSE_ProgressBar.InitSecondBar(0, target.Range.End)
  
  Dim aRange As Word.Range: Set aRange = target.Paragraphs.First.Range
  Do While Not aRange Is Nothing
    If NeedsAlignment(aRange) Then
      Dim firstOnPage As Boolean: firstOnPage = IsFirstInColumn(aRange)
      Dim Headers As HeaderBlock: Headers = FixHeaderBlockSpacing(aRange, firstOnPage)
      Set aRange = Headers.finishRng
    Else
      Set aRange = aRange.Next(wdParagraph, 1)
    End If
    If Not aRange Is Nothing Then _
      Call CSE_ProgressBar.SetB(aRange.End)
  Loop
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Public Function FixGapsStyle(target As Word.Document)
  Dim rFind As Word.Range: Set rFind = target.Range.Duplicate
  With rFind.Find
    .Text = Chr(12)
    Do While .Execute
      If rFind.Previous(wdCharacter, 1) <> Chr(13) Then
        Call rFind.InsertBefore(Chr(13))
        If Not rFind.Start = rFind.Paragraphs.First.Range.Start Then _
          Call rFind.MoveStart(1)
      End If
      rFind.ParagraphFormat.Style = ""
    Loop
  End With
End Function

Public Function CreateLayoutBlock(target As Word.Range)
  If target.End = target.Document.Range.End Then _
    Exit Function

  Dim rStart As Word.Range: Set rStart = target.Duplicate
  Dim rFinish As Word.Range: Set rFinish = target.Next(wdParagraph, 1).Duplicate
  Call rStart.Collapse(wdCollapseStart)
  Call rFinish.Collapse(wdCollapseStart)
  
  If rStart.Start = rFinish.Start Then _
    Exit Function
  If GetColumn(rStart) <> GetColumn(rFinish) Then _
    Exit Function
  
  Dim isFirst As Boolean: isFirst = IsFirstInColumn(target)
  If isFirst Then target.ParagraphFormat.SpaceBefore = 0
  
  Dim prevAfter As Double: prevAfter = rStart.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter
  Dim curBefore As Double: curBefore = rStart.ParagraphFormat.SpaceBefore
  Dim curAfter As Double: curAfter = rFinish.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter
  Dim nxtBefore As Double: nxtBefore = rFinish.ParagraphFormat.SpaceBefore
  
  Dim maxAfter As Double: maxAfter = IIf(curAfter > nxtBefore, curAfter, nxtBefore)
  Dim maxBefore As Double: maxBefore = IIf(curBefore > prevAfter, curBefore, prevAfter)
  
  Call rFinish.Move(wdCharacter, -1)
  Call rFinish.InsertAfter(Chr(13))
  Call rFinish.Move(wdCharacter, 1)
  rFinish.Style = rFinish.Document.Styles(BASE_STYLE)
  
  Dim topPosition As Double: topPosition = rStart.Information(wdVerticalPositionRelativeToPage)
  If IsBorderVisible(rStart, wdBorderTop) Then _
    topPosition = topPosition - BorderLinewidth(wdBorderTop, rStart) + rStart.Borders.DistanceFromTop

  Dim botPosition As Double: botPosition = rFinish.Information(wdVerticalPositionRelativeToPage)
  botPosition = botPosition - rFinish.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter
  
  Dim defSpacing As Double: defSpacing = DefaultSpacing(target.Document)
  Dim blockHeight As Double: blockHeight = botPosition - topPosition
  Dim nLines&: nLines = Int((blockHeight + curBefore + curAfter) / defSpacing + 0.5)
  Dim toAdd As Double: toAdd = (nLines + 1) * defSpacing - (curBefore + curAfter + blockHeight)
  If Abs(toAdd - defSpacing) < 0.5 Then toAdd = 0
  
  rFinish.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter = maxAfter + toAdd / IIf(isFirst, 1, 2)
  rStart.ParagraphFormat.SpaceBefore = maxBefore + toAdd * IIf(isFirst, 0, 0.5)
  If isFirst Then
    rFinish.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter = _
      rFinish.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter + _
      rStart.ParagraphFormat.SpaceBefore
    rStart.ParagraphFormat.SpaceBefore = 0
  End If
  
  rFinish.Delete
  
  Do While rStart.ParagraphFormat.SpaceBefore > 1.4 * defSpacing And _
      (rStart.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter > 1.4 * defSpacing Or _
      rStart.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter = 0)

    rStart.ParagraphFormat.SpaceBefore = _
      rStart.ParagraphFormat.SpaceBefore - defSpacing
    If rStart.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter <> 0 Then _
      rStart.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter = _
        rStart.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter - defSpacing
  Loop

  Do While target.Paragraphs.Last.SpaceAfter > 1.4 * defSpacing And _
      (target.Paragraphs.Last.Next.SpaceBefore > 1.4 * defSpacing Or _
      target.Paragraphs.Last.Next.SpaceBefore = 0)

    target.Paragraphs.Last.SpaceAfter = _
      target.Paragraphs.Last.SpaceAfter - defSpacing
    If target.Paragraphs.Last.Next.SpaceBefore <> 0 Then _
      target.Paragraphs.Last.Next.SpaceBefore = _
        target.Paragraphs.Last.Next.SpaceBefore - defSpacing
  Loop
End Function

Public Function UpdateObjectFields(target As Word.Document)
  Call CSE_ProgressBar.InitSecondBar(0, target.Shapes.Count)
  
  Dim aShape As Word.Shape
  For Each aShape In target.Shapes
    If IsEndSymbol(aShape) Then _
      GoTo NEXT_SHAPE
    Dim ancRange As Word.Range: Set ancRange = aShape.Anchor.Paragraphs(1).Range
    If Len(ancRange) < 3 Then _
      GoTo NEXT_SHAPE
      
    Dim isLeft As Boolean: isLeft = GetColumn(ancRange) = T_COL_LEFT
    With aShape
      '  
      .RelativeVerticalPosition = wdRelativeVerticalPositionParagraph
      .TextFrame.VerticalAnchor = msoAnchorBottom
      .Top = ancRange.ParagraphFormat.SpaceBefore
      If .TextFrame.TextRange.Font.Name = "conceptpict" Then
        .Top = .Top - ancRange.ParagraphFormat.LineSpacing * 1.5 / 2# + .TextFrame.TextRange.ParagraphFormat.LineSpacing / 2#
      Else
        .TextFrame.TextRange.ParagraphFormat.LineSpacing = ancRange.ParagraphFormat.LineSpacing
      End If
      
      Dim prevNormal As Word.Range: Set prevNormal = ancRange.Previous(wdParagraph)
      Dim tmpMin As Double: tmpMin = Min(ancRange.ParagraphFormat.SpaceBefore, prevNormal.ParagraphFormat.SpaceAfter)
      Dim tmpMax As Double: tmpMax = -Min(-ancRange.ParagraphFormat.SpaceBefore, 0)
      .Top = .Top - IIf(ancRange.Sections.First.Range.Paragraphs.First.Range.Start = ancRange.Start, _
              tmpMin, IIf(IsFirstInColumn(ancRange) And isLeft, tmpMax, tmpMin))

      '  
      If .Type = msoTextBox Then
        .RelativeHorizontalPosition = wdRelativeHorizontalPositionColumn
        If isLeft Then
          .Left = CentimetersToPoints(LEFT_POS_SHIFT) - .Width
          .TextFrame.TextRange.ParagraphFormat.Alignment = wdAlignParagraphRight
        Else
          .Left = CentimetersToPoints(RIGHT_POS_SHIFT)
          .TextFrame.TextRange.ParagraphFormat.Alignment = wdAlignParagraphLeft
        End If
        If IsHeader(ancRange) Then _
          .Top = .Top - .Height / 2# _
            + .TextFrame.TextRange.ParagraphFormat.LineSpacing / 2# _
            - .TextFrame.TextRange.Font.Size * (SPACING_SCALE - 1) / 2#
      Else
        .RelativeHorizontalPosition = IIf(isLeft, wdRelativeHorizontalPositionLeftMarginArea, wdRelativeHorizontalPositionRightMarginArea)
        .Left = wdShapeCenter
      End If
    End With
    
NEXT_SHAPE:
    Call CSE_ProgressBar.IncrementB
  Next aShape
  Call CSE_ProgressBar.HideSecondBar
End Function

Public Function UpdateTableOfContents(target As Word.TableOfContents)
  Dim initPos As Word.Range:  Set initPos = target.Application.Selection.Range
  
  Call target.Update
  Dim aLink As Hyperlink
  For Each aLink In target.Range.Hyperlinks
    Call aLink.Follow
    Dim rHeader As Word.Range: Set rHeader = target.Application.Selection.Range
    If rHeader.ParagraphFormat.OutlineLevel = wdOutlineLevel1 Then _
      GoTo NXT_LNK
      
    Call rHeader.MoveStart(wdWord, -1)
    Dim lineStart As Word.Range: Set lineStart = aLink.Range.Duplicate
    Call lineStart.Collapse(wdCollapseStart)
    If rHeader.ShapeRange.Count <> 0 Then
      Dim rSource As Word.Range: Set rSource = rHeader.ShapeRange(1).TextFrame.TextRange
      Call rSource.MoveEnd(wdCharacter, -1)
      lineStart.Text = Chr(9) & rSource.Text & Chr(9)
    Else
      lineStart.Text = Chr(9) & Chr(9)
    End If
    
    Call lineStart.Select
    lineStart.ParagraphFormat.Style.BaseStyle = aLink.Range.ParagraphFormat.Style.BaseStyle
NXT_LNK:
  Next aLink
  
  Call target.Range.Document.ActiveWindow.ScrollIntoView(initPos)
End Function

' =========
Private Function Min(i1 As Variant, i2 As Variant) As Variant
  Min = IIf(i1 < i2, i1, i2)
End Function

Private Function IsEndSymbol(aShape As Word.Shape) As Boolean
  IsEndSymbol = False
  If Abs(aShape.Width - 159.875) > 1 Then Exit Function
  If Abs(aShape.Height - 36) > 1 Then Exit Function
  IsEndSymbol = True
End Function

Private Function FixHeaderBlockSpacing(target As Word.Range, firstOnPage As Boolean) As HeaderBlock
  Call FixHeaderSpacing(target, firstOnPage)
  Dim result As HeaderBlock: result = ScanHeaderBlock(target)
  Dim kScale As Double: kScale = CalculateHeaderScaling(result, firstOnPage)
  Call ScaleHeaderSpacing(result, kScale)
  FixHeaderBlockSpacing = result
End Function

Private Function ScanHeaderBlock(target As Word.Range) As HeaderBlock
  Dim result As HeaderBlock
  
  Set result.startRng = target.Duplicate
  result.yText = target.ParagraphFormat.LineSpacing * target.ComputeStatistics(wdStatisticLines)
  result.yHeight = result.yText + target.ParagraphFormat.SpaceBefore
  
  Set result.finishRng = result.startRng.Next(wdParagraph, 1)
  Do While Not result.finishRng Is Nothing
    If Not IsHeader(result.finishRng) Then _
      Exit Do
    If IsFirstInColumn(result.finishRng) Then _
      Exit Do
      
    Call FixHeaderSpacing(result.finishRng, True)
    
    Dim nextLineSpacing As Double: nextLineSpacing = result.finishRng.ParagraphFormat.LineSpacing
    result.finishRng.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter = nextLineSpacing
    
    Dim textH As Double: textH = nextLineSpacing * result.finishRng.ComputeStatistics(wdStatisticLines)
    result.yText = result.yText + textH
    result.yHeight = result.yHeight + textH + nextLineSpacing
    
    Set result.finishRng = result.finishRng.Next(wdParagraph, 1)
  Loop
  result.finishRng.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter = result.finishRng.ParagraphFormat.LineSpacing
  result.yHeight = result.yHeight + result.finishRng.ParagraphFormat.LineSpacing
  
  ScanHeaderBlock = result
End Function

Private Function CalculateHeaderScaling(block As HeaderBlock, firstOnPage As Boolean) As Double
  Dim defSpacing As Double: defSpacing = DefaultSpacing(block.startRng.Document)
  Dim n&: n = CLng(block.yHeight / defSpacing + 0.5) ' TODO: probably should use Int istead of CLng, needs testing
  If n = 3 And Not firstOnPage Then
    n = n - 1
  End If
  Dim newH As Double: newH = n * defSpacing
  Dim k As Double: k = (newH - block.yText) / (block.yHeight - block.yText)
  If k < 0 Then
    Call UserInteraction.ShowMessage(EM_FIX_LINING_FAIL)
    Exit Function
  End If
  CalculateHeaderScaling = k
End Function

Private Function ScaleHeaderSpacing(block As HeaderBlock, kScale As Double)
  Dim tmpRng As Word.Range: Set tmpRng = block.startRng.Duplicate
  Do While Not tmpRng.Start = block.finishRng.Start
    tmpRng.ParagraphFormat.SpaceBefore = kScale * tmpRng.ParagraphFormat.SpaceBefore
    tmpRng.ParagraphFormat.SpaceAfter = kScale * tmpRng.ParagraphFormat.SpaceAfter
    Set tmpRng = tmpRng.Next(wdParagraph, 1)
  Loop
End Function

Private Function SetHeaderBlockPosition(block As HeaderBlock, firstOnPage As Boolean)
  Dim defSpacing As Double: defSpacing = DefaultSpacing(block.startRng.Document)
  Dim prevSpc As Double:  prevSpc = block.startRng.Previous(wdParagraph).ParagraphFormat.SpaceAfter
  Dim curSpc As Double:   curSpc = block.startRng.ParagraphFormat.SpaceBefore
  If prevSpc <> 0 And Not firstOnPage Then _
    block.startRng.ParagraphFormat.SpaceBefore = prevSpc + ((curSpc * 1000) Mod (defSpacing * 1000)) / 1000
  
  prevSpc = block.finishRng.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter
  curSpc = block.finishRng.ParagraphFormat.SpaceBefore
  If curSpc <> 0 Then _
    block.finishRng.Previous(wdParagraph, 1).ParagraphFormat.SpaceAfter = curSpc + ((1000 * prevSpc) Mod (defSpacing * 1000)) / 1000
End Function

Private Function FixHeaderSpacing(target As Word.Range, columnStart As Boolean)
  With target
    If Not .ParagraphFormat.LineSpacingRule = wdLineSpaceExactly Then
      .ParagraphFormat.LineSpacingRule = wdLineSpaceExactly
      .ParagraphFormat.LineSpacing = .ParagraphFormat.LineSpacing * SPACING_SCALE
    End If
    .ParagraphFormat.Alignment = IIf(GetColumn(target) = T_COL_RIGHT, wdAlignParagraphRight, wdAlignParagraphLeft)
    .ParagraphFormat.SpaceBefore = IIf(columnStart, 0, .ParagraphFormat.LineSpacing)
  End With
End Function

Private Function BorderLinewidth(bordertype As WdBorderType, wdrange As Word.Range) As Double
  BorderLinewidth = 0
  If wdrange.Borders(bordertype).Visible = False Then Exit Function
  Select Case wdrange.Borders(bordertype).LineWidth
    Case wdLineWidth025pt:  BorderLinewidth = 0.25
    Case wdLineWidth050pt:  BorderLinewidth = 0.5
    Case wdLineWidth075pt:  BorderLinewidth = 0.75
    Case wdLineWidth100pt:  BorderLinewidth = 1
    Case wdLineWidth150pt:  BorderLinewidth = 1.5
    Case wdLineWidth225pt:  BorderLinewidth = 2.25
    Case wdLineWidth300pt:  BorderLinewidth = 3
    Case wdLineWidth450pt:  BorderLinewidth = 4.5
    Case wdLineWidth600pt:  BorderLinewidth = 6
  End Select
End Function

Private Function IsBorderVisible(wdrange As Word.Range, wdBorder As WdBorderType) As Boolean
  IsBorderVisible = wdrange.Borders.Item(wdBorder).Visible
End Function

Private Function RemoveObjectsFrom(dest As Word.Range)
  Dim i&
  For i = 1 To dest.ShapeRange.Count
    Dim aShape As Word.Shape: Set aShape = dest.ShapeRange(i)
    If aShape.Type = msoTextBox And ((GetColumn(dest) = T_COL_RIGHT And aShape.Left = Int(20 * CentimetersToPoints(RIGHT_POS_SHIFT)) / 20) _
      Or (GetColumn(dest) = T_COL_LEFT And Abs(aShape.Left) = Int(20 * Abs(CentimetersToPoints(LEFT_POS_SHIFT) - aShape.Width)) / 20)) _
    Then
      Call aShape.Delete
    End If
  Next i
End Function

Private Function NeedsAlignment(target As Word.Range) As Boolean
  NeedsAlignment = False
  
  Dim outLevel&: outLevel = target.ParagraphFormat.OutlineLevel
  If IsHeader(target, 1) Or outLevel = wdOutlineLevel1 Then _
    Exit Function
  If Not IsHeader(target) Then _
    Exit Function
  If target.ParagraphFormat.Style = " " Then _
    Exit Function
  
  NeedsAlignment = True
End Function

Private Function CreateEmptyField(dest As Word.Range) As Word.Shape
  Dim selRange As Word.Range: Set selRange = dest.Application.Selection.Range
  
  Call RemoveObjectsFrom(dest)
  
  Dim txtShp As Word.Shape
  Set txtShp = dest.Document.Shapes.AddTextbox(msoTextOrientationHorizontal, 0, 0, _
    CentimetersToPoints(FIELD_SIZE_CM), dest.ParagraphFormat.LineSpacing * 1.5, dest)
  
  If txtShp.Anchor.Start <> dest.Start Then
    ' :   
    txtShp.Select
    Selection.Cut
    dest.Select
    Selection.Paste
    Set txtShp = dest.ShapeRange(1)
    selRange.Select
    If dest.Characters.First = " " Then _
      dest.Characters.First.Delete
    ' End of 
  End If
  
  With txtShp
    '  
    With .TextFrame.TextRange
      .Font = dest.Font
      .ParagraphFormat.LineSpacingRule = wdLineSpaceExactly
      .ParagraphFormat.LineSpacing = dest.ParagraphFormat.LineSpacing
      .ParagraphFormat.SpaceBefore = 0
      .ParagraphFormat.SpaceAfter = 0
    End With
    
    '   
    .RelativeVerticalPosition = wdRelativeVerticalPositionParagraph
    .Top = dest.ParagraphFormat.SpaceBefore _
        - .Height / 2# _
        + .TextFrame.TextRange.ParagraphFormat.LineSpacing / 2#
    
    .RelativeHorizontalPosition = wdRelativeHorizontalPositionColumn
    If GetColumn(dest) = T_COL_RIGHT Then
      .Left = CentimetersToPoints(RIGHT_POS_SHIFT)
      .TextFrame.TextRange.ParagraphFormat.Alignment = wdAlignParagraphLeft
    Else
      .Left = CentimetersToPoints(LEFT_POS_SHIFT) - .Width
      .TextFrame.TextRange.ParagraphFormat.Alignment = wdAlignParagraphRight
    End If
    
    ' 
    .Line.Visible = msoFalse
  End With
  Set CreateEmptyField = txtShp
End Function

