Attribute VB_Name = "CD_Paint"
Option Private Module
Option Explicit

Public Function RepaintText(target As Word.Document)
  CSE_ProgressBar.Header = " ..."
  Call CSE_ProgressBar.InitSecondBar(0, maxVal:=target.Range.End)
  
  Dim sectionHead As SectionHeader: Set sectionHead.rFind = target.Range.Duplicate
  sectionHead.nStart = 0
  sectionHead.nFinish = -1
  sectionHead.cHeader = wdAuto
  sectionHead.cText = wdAuto
  With sectionHead.rFind.Find
    .Text = ""
    .Format = True
    .ParagraphFormat.OutlineLevel = wdOutlineLevel1
  End With
  
  Do While IncrementHeader(sectionHead)
    Call PaintBoldText(sectionHead)
    Call PaintItalicText(sectionHead)
    Call CSE_ProgressBar.SetB(sectionHead.nFinish)
  Loop

  Call CSE_ProgressBar.HideSecondBar
End Function

Public Function RepaintTextShapes(target As Word.Document)
  CSE_ProgressBar.Header = "   ..."
  Call CSE_ProgressBar.InitSecondBar(0, maxVal:=target.Shapes.Count)
  
  Dim aShape As Word.Shape
  For Each aShape In target.Shapes
    If aShape.Type = msoTextBox Then
      Dim ancRange As Word.Range: Set ancRange = aShape.Anchor.Paragraphs(1).Range
      aShape.TextFrame.TextRange.Font.Color = GetSectionHeader(ancRange.Start, target).Font.Color
    End If
    Call CSE_ProgressBar.IncrementB
  Next aShape
  
  Call CSE_ProgressBar.HideSecondBar
End Function

Public Function RepaintHLinks(target As Word.Document)
  CSE_ProgressBar.Header = " ..."
  Call CSE_ProgressBar.InitSecondBar(0, maxVal:=target.Hyperlinks.Count)
  
  Dim initialPos As Word.Range:  Set initialPos = target.ActiveWindow.Selection.Range
  Dim aLink As Hyperlink
  For Each aLink In target.Hyperlinks
    If Not aLink.Address = "" Or aLink.SubAddress Like "_Toc*" Then _
      GoTo NEXT_LINK
      
    Call aLink.Follow
    Dim linkTarget As Word.Range: Set linkTarget = target.ActiveWindow.Selection.Paragraphs(1).Range
    If linkTarget.ParagraphFormat.OutlineLevel <> wdOutlineLevelBodyText Then _
      aLink.Range.Font.Color = linkTarget.Font.Color
NEXT_LINK:
  Call CSE_ProgressBar.IncrementB
  Next aLink
  
  Call target.ActiveWindow.ScrollIntoView(initialPos)
  Call CSE_ProgressBar.HideSecondBar
End Function

Public Function RepaintToC(target As Word.Document)
  CSE_ProgressBar.Header = " ..."
  Dim i&
  For i = 1 To target.TablesOfContents.Count Step 1
    Call PaintToC(target.TablesOfContents(i))
  Next i
End Function

Public Function PaintTable(target As Word.Table, mainColor&, props As TablePaintProps)
  Dim colorFrame&: colorFrame = ColorAdjustLuma(mainColor, props.cGrid, True)
  Dim colorHead&: colorHead = ColorAdjustLuma(mainColor, props.cHeading, True)
  Dim colorSubhead&: colorSubhead = ColorAdjustLuma(mainColor, props.cSubHead, True)
  Dim colorZebraBright&: colorZebraBright = 16777215 ' 
  Dim colorZebraDark&: colorZebraDark = ColorAdjustLuma(mainColor, props.cZebra, True)
  
  Dim greyColHead&:  greyColHead = ColorGetLuma(colorHead)
  Dim greyColSub&:  greyColSub = ColorGetLuma(colorSubhead)
  Dim greyColZebra&:  greyColZebra = ColorGetLuma(colorZebraDark)
  
  With target
  '  
    .AllowPageBreaks = True
    .AllowAutoFit = False
        
    .PreferredWidthType = wdPreferredWidthPercent
    .PreferredWidth = 100
    
    .Rows.HeightRule = wdRowHeightAuto
    .Rows.Height = CentimetersToPoints(0)
    
    .TopPadding = CentimetersToPoints(props.textSpacing)
    .BottomPadding = CentimetersToPoints(props.textSpacing)
    .LeftPadding = CentimetersToPoints(props.textSpacing)
    .RightPadding = CentimetersToPoints(props.textSpacing)
    .Spacing = 0
    
  ' 
    With .Borders(wdBorderLeft)
      .LineStyle = wdLineStyleSingle
      .LineWidth = TABLE_FRAME_THICKNESS
      .Color = colorFrame
    End With
    With .Borders(wdBorderTop)
      .LineStyle = wdLineStyleSingle
      .LineWidth = TABLE_FRAME_THICKNESS
      .Color = colorFrame
    End With
    With .Borders(wdBorderRight)
      .LineStyle = wdLineStyleSingle
      .LineWidth = TABLE_FRAME_THICKNESS
      .Color = colorFrame
    End With
    With .Borders(wdBorderBottom)
      .LineStyle = wdLineStyleSingle
      .LineWidth = TABLE_FRAME_THICKNESS
      .Color = colorFrame
    End With
    
    With .Borders(wdBorderHorizontal)
      .LineStyle = wdLineStyleSingle
      .LineWidth = TABLE_CELL_THICKNESS
      .Color = colorFrame
    End With
    With .Borders(wdBorderVertical)
      .LineStyle = wdLineStyleSingle
      .LineWidth = TABLE_CELL_THICKNESS
      .Color = colorFrame
    End With
    
    
    With .Rows(1)
      With .Borders(wdBorderBottom)
        .LineStyle = wdLineStyleSingle
        .LineWidth = TABLE_FRAME_THICKNESS
        .Color = colorFrame
      End With
      
      .Range.Font.ColorIndex = IIf(greyColHead < 127, wdWhite, wdBlack)
      .Shading.BackgroundPatternColor = colorHead
    End With
  End With
  
  '  
  Dim whiteFlag As Boolean: whiteFlag = True
  Dim nRow As Integer: nRow = 2
  For nRow = 2 To target.Rows.Count
    If target.Cell(nRow, 1).Range.Font.Bold = True Then
      target.Rows(nRow).Range.Font.ColorIndex = IIf(greyColSub < 127, wdWhite, wdBlack)
      target.Rows(nRow).Shading.BackgroundPatternColor = colorSubhead
      whiteFlag = True
    Else
      If whiteFlag Then
        target.Rows(nRow).Shading.BackgroundPatternColor = colorZebraBright
      Else
        target.Rows(nRow).Range.Font.ColorIndex = IIf(greyColZebra < 127, wdWhite, wdBlack)
        target.Rows(nRow).Shading.BackgroundPatternColor = colorZebraDark
      End If
      whiteFlag = Not whiteFlag
    End If
  Next nRow
End Function

Public Function PaintTableProto(target As Word.Table, proto As Word.Table)
'       
  Dim colorHead&: colorHead = proto.Cell(2, 1).Shading.BackgroundPatternColor
  Dim colorBright&: colorBright = proto.Cell(3, 1).Shading.BackgroundPatternColor
  Dim colorDark&: colorDark = proto.Cell(4, 1).Shading.BackgroundPatternColor
  
  '  
  With target
    With .Borders(wdBorderLeft)
      .LineStyle = proto.Borders(wdBorderLeft).LineStyle
      .LineWidth = proto.Borders(wdBorderLeft).LineWidth
      .Color = proto.Borders(wdBorderLeft).Color
    End With
    With .Borders(wdBorderTop)
      .LineStyle = proto.Borders(wdBorderTop).LineStyle
      .LineWidth = proto.Borders(wdBorderTop).LineWidth
      .Color = proto.Borders(wdBorderTop).Color
    End With
    With .Borders(wdBorderRight)
      .LineStyle = proto.Borders(wdBorderRight).LineStyle
      .LineWidth = proto.Borders(wdBorderRight).LineWidth
      .Color = proto.Borders(wdBorderRight).Color
    End With
    With .Borders(wdBorderBottom)
      .LineStyle = proto.Borders(wdBorderBottom).LineStyle
      .LineWidth = proto.Borders(wdBorderBottom).LineWidth
      .Color = proto.Borders(wdBorderBottom).Color
    End With
    
    With .Borders(wdBorderHorizontal)
      .LineStyle = proto.Borders(wdBorderVertical).LineStyle
      .LineWidth = proto.Borders(wdBorderVertical).LineWidth
      .Color = proto.Borders(wdBorderVertical).Color
    End With
    With .Borders(wdBorderVertical)
      .LineStyle = proto.Borders(wdBorderVertical).LineStyle
      .LineWidth = proto.Borders(wdBorderVertical).LineWidth
      .Color = proto.Borders(wdBorderVertical).Color
    End With
    
    .Borders(wdBorderDiagonalDown).LineStyle = proto.Borders(wdBorderDiagonalDown).LineStyle
    .Borders(wdBorderDiagonalUp).LineStyle = proto.Borders(wdBorderDiagonalUp).LineStyle
    .Borders.Shadow = proto.Borders.Shadow
  End With
  
  With target
    .AllowPageBreaks = proto.AllowPageBreaks
    .AllowAutoFit = proto.AllowAutoFit
    
    .PreferredWidthType = wdPreferredWidthPoints
    .PreferredWidth = CentimetersToPoints(15.92)
    .Rows.HeightRule = wdRowHeightAuto
    .Rows.Height = CentimetersToPoints(0)
    
    .TopPadding = proto.TopPadding
    .BottomPadding = proto.BottomPadding
    .LeftPadding = proto.LeftPadding
    .RightPadding = proto.RightPadding
    .Spacing = proto.Spacing
    
    With .Rows(1)
      .Borders(wdBorderBottom).LineStyle = proto.Rows(1).Borders(wdBorderBottom).LineStyle
      .Borders(wdBorderBottom).LineWidth = proto.Rows(1).Borders(wdBorderBottom).LineWidth
      .Borders(wdBorderBottom).Color = proto.Rows(1).Borders(wdBorderBottom).Color
      .Shading.BackgroundPatternColor = proto.Cell(1, 1).Shading.BackgroundPatternColor
    End With
  End With
  
  '  
  Dim whiteFlag As Boolean: whiteFlag = True
  Dim nRow&: nRow = 2
  For nRow = 2 To target.Rows.Count
    If target.Cell(nRow, 1).Range.Font.Bold = True Then
      target.Rows(nRow).Shading.BackgroundPatternColor = colorHead
      whiteFlag = True
    Else
      If whiteFlag Then
        target.Rows(nRow).Shading.BackgroundPatternColor = colorBright
      Else
        target.Rows(nRow).Shading.BackgroundPatternColor = colorDark
      End If
      whiteFlag = Not whiteFlag
    End If
  Next nRow
End Function

' ==========
Private Function PaintToC(target As Word.TableOfContents)
  Dim tocElement As Word.Range: Set tocElement = target.Range.Paragraphs.First.Range
  Dim rHeader As Word.Range: Set rHeader = target.Range.Document.Range
  With rHeader.Find
    .Text = ""
    .Format = True
    .ParagraphFormat.OutlineLevel = wdOutlineLevel1
  End With
  
  On Error GoTo REPORT_ERROR
  Dim theEnd&: theEnd = target.Range.End
  Do While rHeader.Find.Execute
    If Len(rHeader.Text) < 3 Then _
      GoTo NEXT_SECTION_HEADER
    
    Do While Not tocElement.Font.AllCaps = True
      Set tocElement = tocElement.Next(wdParagraph, 1)
      If tocElement.End > theEnd Then _
        GoTo REPORT_ERROR
    Loop
    tocElement.Font.Color = rHeader.Font.Color
    Set tocElement = tocElement.Next(wdParagraph, 1)
    
NEXT_SECTION_HEADER:
    Call rHeader.Collapse(wdCollapseEnd)
  Loop
  Exit Function
REPORT_ERROR:
  Call UserInteraction.ShowMessage(EM_INVALID_CONTENTS_TABLE)
End Function

Private Function IncrementHeader(ByRef target As SectionHeader) As Boolean
  IncrementHeader = False
  If target.nFinish >= target.rFind.Document.Sections.Last.Range.Start - 1 Then _
    Exit Function
  If Not target.rFind.Find.Execute Then _
    Exit Function
    
  Dim theHeader As Word.Range: Set theHeader = target.rFind.Duplicate
  Dim lastSection As Boolean: lastSection = Not target.rFind.Find.Execute
  Do While Len(target.rFind.Text) < 3 And Not lastSection
    Call target.rFind.Collapse(wdCollapseEnd)
    lastSection = Not target.rFind.Find.Execute
  Loop
  
  Call target.rFind.Collapse(wdCollapseStart)
  target.nStart = theHeader.End + 1
  target.nFinish = IIf(lastSection, target.rFind.Document.Sections.Last.Range.Start - 1, target.rFind.Start - 1)
  target.cText = ColorGetRGB(theHeader.Font.Color, target.rFind.Document)
  target.cHeader = ColorAdjustLuma(target.cText, 1000 * TEXT_BOLD_LUMSCALE, True)
  
  IncrementHeader = True
End Function

Private Function PaintBoldText(target As SectionHeader)
  Dim rFind As Word.Range: Set rFind = target.rFind.Document.Range(target.nStart, target.nStart)
  With rFind
    .Find.Text = ""
    .Find.Format = True
    .Find.Font.Bold = True
    
    Do While .Find.Execute And .End < target.nFinish
      If .OMaths.Count > 0 Or .Tables.Count > 0 Then _
        GoTo NEXT_RANGE
      If .Style Like "**" Then _
        GoTo NEXT_RANGE
      If Not .CharacterStyle Is Nothing Then
        If .CharacterStyle Like "**" Then _
          GoTo NEXT_RANGE
      End If
      
      .Font.Color = IIf(.ParagraphFormat.OutlineLevel = wdOutlineLevelBodyText, target.cText, target.cHeader)
      
NEXT_RANGE:
      Call .Collapse(wdCollapseEnd)
    Loop
  End With
End Function

Private Function PaintItalicText(target As SectionHeader)
  Dim rFind As Word.Range: Set rFind = target.rFind.Document.Range(target.nStart, target.nStart)
  With rFind
    .Find.Text = ""
    .Find.Format = True
    .Find.Font.Italic = True
    
    Do While .Find.Execute And .End < target.nFinish
      If .OMaths.Count > 0 Or .Tables.Count > 0 Then _
        GoTo NEXT_RANGE
      If .Style Like "**" Then _
        GoTo NEXT_RANGE
      If Not .CharacterStyle Is Nothing Then
        If .CharacterStyle Like "**" Then _
          GoTo NEXT_RANGE
      End If
    
      .Font.Color = IIf(.ParagraphFormat.OutlineLevel = wdOutlineLevelBodyText, target.cText, target.cHeader)
      
NEXT_RANGE:
      Call .Collapse(wdCollapseEnd)
    Loop
  End With
End Function
