Attribute VB_Name = "CD_SplitTable"
Option Private Module
Option Explicit

Public Function PrepareTableForSplit(target As Word.Table)
  Call SetupPageBeforeSlice(target)
  Call InsertSplittingMarker(target)
End Function

Public Function RemoveSplitMarkerFrom(target As Word.Table)
  On Error Resume Next
  target.Cell(1, 1).Range.ShapeRange.Delete
End Function

Public Function SliceTable(target As Word.Table) As Collection
  Dim startingRows As Collection: Set startingRows = PrepareStartingRows(target)

  Dim slices As New Collection
  Dim nIndex&
  For nIndex = startingRows.Count To 2 Step -1
    Call target.Rows(1).Select
    target.Application.Selection.Copy
    Dim rPaste As Word.Range: Set rPaste = target.Range
    Call rPaste.Collapse(wdCollapseEnd)
    Call rPaste.InsertBreak(wdSectionBreakNextPage)
    Call rPaste.Paste
    
    Dim rowStart&: rowStart = startingRows(nIndex)
    Dim rowFinish&: rowFinish = target.Rows.Count + 1
    target.Rows(rowStart).Select
    Dim k&
    For k = 1 To rowFinish - rowStart - 1
      Call Selection.MoveDown(wdLine, Extend:=wdExtend)
    Next k
    
    target.Application.Selection.Copy
    Call rPaste.Collapse(wdCollapseEnd)
    Call rPaste.Paste
    target.Application.Selection.Rows.Delete
    
    Call slices.Add(rPaste.Sections.First.Range.Tables(1))
  Next nIndex
  Call slices.Add(target)
  
  Set SliceTable = slices
End Function

Public Function SplitSlices(slices As Collection)
  Call CSE_ProgressBar.InitSecondBar(0, slices.Count)
  Dim tablePart As Variant
  For Each tablePart In slices
    Call tablePart.Select
    Call SplitTable(tablePart.Application.Selection.Tables(1))
    CSE_ProgressBar.IncrementB
  Next tablePart
  CSE_ProgressBar.HideSecondBar
End Function

' ============
Private Function SplitTable(target As Word.Table)
  If target.Rows(1).Cells.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_TABLE_MERGED_CELLS)
    Exit Function
  End If
  
  Dim horiz As Double, vert As Double
  Dim tmpRange As Word.Range
  Set tmpRange = target.Application.Selection.Document.Range(target.Range.End - 1, target.Range.End - 1)
  horiz = tmpRange.Information(wdHorizontalPositionRelativeToPage)
  vert = tmpRange.Information(wdVerticalPositionRelativeToPage)
  
  Dim runner As Variant
  If target.Rows(1).Cells.Count < 2 Then
    Call UserInteraction.ShowMessage(EM_INVALID_TABLE_COLUMNS)
    Exit Function
  End If
  
  For Each runner In target.Rows(1).Cells
    Set tmpRange = runner.Range
    Call tmpRange.Collapse(wdCollapseStart)
    If PointsToCentimeters(tmpRange.Information(wdHorizontalPositionRelativeToPage)) > TABLE_CRITICAL_WIDTH Then
      runner.Select
      GoTo SELECT_MOVE
    End If
  Next runner
  
SELECT_MOVE:
  Set tmpRange = target.Application.Selection.Range
  Call tmpRange.Collapse(wdCollapseEnd)
  
  Do While tmpRange.Information(wdHorizontalPositionRelativeToPage) <> horiz
    Call target.Application.Selection.MoveRight(wdCharacter, Extend:=wdExtend)
    Set tmpRange = target.Application.Selection.Range
    Call tmpRange.Collapse(wdCollapseEnd)
  Loop
  
  Do While tmpRange.Information(wdVerticalPositionRelativeToPage) <> vert Or tmpRange.End <> target.Range.End - 1
    Call Selection.MoveDown(wdLine, Extend:=wdExtend)
    Set tmpRange = target.Application.Selection.Range
    Call tmpRange.Collapse(wdCollapseEnd)
  Loop
  
  Do While tmpRange.Information(wdHorizontalPositionRelativeToPage) <> horiz
    Call target.Application.Selection.MoveRight(wdCharacter, Extend:=wdExtend)
    Set tmpRange = target.Application.Selection.Range
    Call tmpRange.Collapse(wdCollapseEnd)
  Loop
  
  Set tmpRange = target.Range
  Call tmpRange.Collapse(wdCollapseEnd)

  Call tmpRange.Collapse(wdCollapseEnd)
  Call tmpRange.InsertBreak(wdSectionBreakNextPage)
  tmpRange.Collapse (wdCollapseEnd)
  
  Dim nmeRng As Word.Range
  Set nmeRng = target.Range.Document.Range(target.Range.Sections(1).Range.Start, target.Range.Start)
  
  Dim beforeTable As Double: beforeTable = TabNameHeight(target)
  If beforeTable > 0 Then
    Call nmeRng.Copy
    tmpRange.Paste
    tmpRange.Collapse (wdCollapseEnd)
    nmeRng.Font.ColorIndex = wdWhite
  End If
  
  target.Application.Selection.Copy
  tmpRange.Paste
  target.Application.Selection.Columns.Delete
  
  Dim curSec As Word.Section, nextSec As Word.Section
  Set curSec = target.Range.Sections.First
  Set nextSec = tmpRange.Sections.First
  
  Call DoPortraitPageSetup(curSec.PageSetup)
  Call DoPortraitPageSetup(nextSec.PageSetup)
  
  target.PreferredWidthType = wdPreferredWidthPoints
  target.PreferredWidth = CentimetersToPoints(21 - 2 * FIELD_SIZE_CM)
  
  Dim victim As Word.Table
  Set victim = nextSec.Range.Tables(1)
  victim.PreferredWidthType = wdPreferredWidthPoints
  victim.PreferredWidth = CentimetersToPoints(21 - 2 * FIELD_SIZE_CM)
  
  Dim tSecBot As Double, vSecBot As Double
  tSecBot = target.Range.Sections(1).PageSetup.BottomMargin
  vSecBot = victim.Range.Sections(1).PageSetup.BottomMargin
  
  target.Range.Sections(1).PageSetup.BottomMargin = tSecBot / 4
  victim.Range.Sections(1).PageSetup.BottomMargin = vSecBot / 4
  Dim i As Integer, tH As Double, vH As Double
  For i = 1 To target.Rows.Count Step 1
    tH = Badheight(target.Rows(i))
    vH = Badheight(victim.Rows(i))
    target.Rows(i).HeightRule = wdRowHeightExactly
    victim.Rows(i).HeightRule = wdRowHeightExactly
    target.Rows(i).Height = IIf(tH > vH, tH, vH) - 3
    victim.Rows(i).Height = IIf(tH > vH, tH, vH) - 3
  Next i
  
  target.Range.Sections(1).PageSetup.BottomMargin = tSecBot
  victim.Range.Sections(1).PageSetup.BottomMargin = vSecBot
  
  If target.Cell(1, 1).Range.Information(wdActiveEndPageNumber) <> _
    target.Rows.Last.Cells(1).Range.Information(wdActiveEndPageNumber) Then
    target.Rows.Last.HeightRule = wdRowHeightAuto
  End If

  If victim.Cell(1, 1).Range.Information(wdActiveEndPageNumber) <> _
    victim.Rows.Last.Cells(1).Range.Information(wdActiveEndPageNumber) Then
    victim.Rows.Last.HeightRule = wdRowHeightAuto
  End If
End Function

Private Function RowHeight(wdrow As Word.Row) As Double
  Dim tmpCell As Variant
  Dim jumper As Word.Range, strt As Double, ent As Double, tmpMax As Double
  Dim bln As Boolean: bln = Application.ScreenUpdating
  Application.ScreenUpdating = False
  For Each tmpCell In wdrow.Cells
    Set jumper = tmpCell.Range
    Set jumper = jumper.Document.Range(jumper.End - 1, jumper.End - 1)
    jumper.InsertAfter (Chr(11))
    
    Set jumper = tmpCell.Range
    Set jumper = jumper.Document.Range(jumper.Start, jumper.Start)
    strt = jumper.Information(wdVerticalPositionRelativeToPage)
    Set jumper = tmpCell.Range
    Set jumper = jumper.Document.Range(jumper.End - 1, jumper.End - 1)
    ent = jumper.Information(wdVerticalPositionRelativeToPage)
    
    Set jumper = tmpCell.Range
    Set jumper = jumper.Document.Range(jumper.End - 2, jumper.End - 1)
    jumper.Delete
    tmpMax = IIf(tmpMax > ent - strt + jumper.ParagraphFormat.SpaceAfter, tmpMax, ent - strt + jumper.ParagraphFormat.SpaceAfter)
  Next tmpCell
  Application.ScreenUpdating = bln
  RowHeight = tmpMax
End Function

Private Function Badheight(wdrow As Word.Row) As Double
  Dim strt As Double, ent As Double
  strt = wdrow.Cells(1).Range.Information(wdVerticalPositionRelativeToPage)
  ent = InsertFictiveAfter(wdrow)
  Badheight = IIf(ent > strt, ent - strt, CentimetersToPoints(TABLE_CRITICAL_HEIGHT) - strt)
End Function

Private Function InsertFictiveAfter(wdrow As Word.Row) As Double
  wdrow.Select
  With wdrow.Application.Selection
    .InsertRowsBelow (1)
    .Font.Size = 1
    .Font.Scaling = 1
    .Rows.HeightRule = wdRowHeightExactly
    .Rows.Height = 0
    InsertFictiveAfter = .Range.Information(wdVerticalPositionRelativeToPage)
    Selection.Rows.Delete
  End With
End Function

Private Function TabNameHeight(wdTable As Word.Table) As Double
  If wdTable.Range.Start = wdTable.Range.Sections.First.Range.Start Then
    TabNameHeight = -1
    Exit Function
  End If

  Dim nameRange As Word.Range
  With wdTable.Range
    Set nameRange = .Document.Range(.Sections.First.Range.Start, .Start)
  End With
  
  Dim tmpRange As Word.Range
  Dim cumHeight As Double
  With nameRange
    
    Set tmpRange = .Duplicate
    Call tmpRange.Collapse(wdCollapseStart)
    cumHeight = tmpRange.Information(wdVerticalPositionRelativeToPage) - tmpRange.ParagraphFormat.SpaceBefore
    
    Set tmpRange = .Duplicate
    Call tmpRange.Collapse(wdCollapseEnd)
    cumHeight = tmpRange.Information(wdVerticalPositionRelativeToPage) _
      - tmpRange.ParagraphFormat.SpaceBefore - cumHeight - 3
  End With
  
  TabNameHeight = IIf(cumHeight < 0, 0, cumHeight)
End Function

' ============
Private Function SetupPageBeforeSlice(target As Word.Table)
  Call DoPortraitPageSetup(target.Range.Sections.First.PageSetup)
  With target.Range.Sections.First.PageSetup
    .PageWidth = 2 * .PageWidth - 2 * .LeftMargin
    target.PreferredWidthType = wdPreferredWidthPoints
    target.PreferredWidth = .PageWidth - .LeftMargin - .RightMargin
  End With
End Function

Private Function DoPortraitPageSetup(ByRef pSetup As PageSetup)
  With pSetup
    .Orientation = wdOrientPortrait
    .PageHeight = CentimetersToPoints(29.7)
    .PageWidth = CentimetersToPoints(21)
    .TopMargin = CentimetersToPoints(FIELD_SIZE_CM)
    .BottomMargin = .TopMargin
    .LeftMargin = CentimetersToPoints(FIELD_SIZE_CM)
    .RightMargin = .LeftMargin
  End With
End Function

Private Function PrepareStartingRows(target As Word.Table) As Collection
  Dim startingRows As New Collection
  Dim previousY As Double: previousY = -1
  Dim nRow&
  For nRow = 1 To target.Rows.Count
    Dim currentY As Double: currentY = target.Rows(nRow).Cells(1).Range.Information(wdVerticalPositionRelativeToPage)
    If currentY < previousY Or Badheight(target.Rows(nRow - 1)) > CentimetersToPoints(TABLE_CRITICAL_HEIGHT) Then
      Call startingRows.Add(nRow)
    End If
    previousY = currentY
  Next nRow
  
  Set PrepareStartingRows = startingRows
End Function

Private Function InsertSplittingMarker(target As Word.Table)
  Dim pSetup As PageSetup: Set pSetup = target.Range.Sections.First.PageSetup
  Dim center As Double: center = (pSetup.PageWidth - 2 * pSetup.TopMargin) / 2
  Dim firstCell As Word.Cell: Set firstCell = target.Cell(1, 1)
  
  Call firstCell.Select
  Dim sMarker As Word.Shape
  Set sMarker = target.Parent.Shapes.AddShape(msoShapeDownArrow, center - 15, 0, 30, 60)
  sMarker.Fill.ForeColor.RGB = RGB(112, 48, 160)
  sMarker.Select
  target.Application.Selection.Cut
  Call firstCell.Range.Characters.First.Paste
  
  Dim markerAncore As Word.ShapeRange: Set markerAncore = firstCell.Range.ShapeRange
  With markerAncore
    .RelativeHorizontalPosition = wdRelativeHorizontalPositionPage
    .RelativeVerticalPosition = wdRelativeVerticalPositionPage
    .Left = center - 15
    .Top = -pSetup.TopMargin
  End With
End Function
